local headSlot = GetInventorySlotInfo("HeadSlot")
local backSlot = GetInventorySlotInfo("BackSlot")

local sortedItems = {
	helm = {},
	cloak = {},
}

local equipSlots = {
	INVTYPE_HEAD = "helm",
	INVTYPE_CLOAK = "cloak",
}

local tabIDs = {
	"helm",
	"cloak",
}

local helmID, cloakID

local NUM_BUTTONS = 12
local BUTTON_HEIGHT = 20

local addon = CreateFrame("Frame")
addon:RegisterEvent("ADDON_LOADED")
addon:RegisterEvent("PLAYER_LOGIN")
addon:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
addon:RegisterEvent("TRANSMOGRIFY_SUCCESS")
addon:SetScript("OnEvent", function(self, event, ...)
	return self[event] and self[event](self, ...)
end)

local frame = CreateFrame("Frame", "FashionFrame", UIParent)
frame:EnableMouse(true)
frame:SetMovable(true)
frame:SetToplevel(true)
frame:RegisterForDrag("LeftButton")
frame:SetPoint("CENTER")
frame:SetSize(384, NUM_BUTTONS * BUTTON_HEIGHT + 58 + 16)
frame:SetBackdrop({
	bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
	edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
	tile = true, tileSize = 16, edgeSize = 16,
	insets = {left = 4, right = 4, top = 4, bottom = 4}
})
frame:SetBackdropColor(0.1, 0.1, 0.1, 1)
frame:SetBackdropBorderColor(0.3, 0.3, 0.3, 1)
frame:Hide()
frame:SetScript("OnDragStart", frame.StartMoving)
frame:SetScript("OnDragStop", frame.StopMovingOrSizing)
frame:SetScript("OnHide", frame.StopMovingOrSizing)

-- scroll frame with database items
local scrollFrame = CreateFrame("ScrollFrame", "FashionItemsScrollFrame", frame, "FauxScrollFrameTemplate")
scrollFrame:SetPoint("TOPLEFT", 12, -44)
scrollFrame:SetPoint("BOTTOMRIGHT", -38, 32)
scrollFrame:SetScript("OnShow", function(self)
	FauxScrollFrame_SetOffset(self, 0)
	self.ScrollBar:SetValue(0)
	self:Update()
end)
scrollFrame:SetScript("OnVerticalScroll", function(self, offset) FauxScrollFrame_OnVerticalScroll(self, offset, BUTTON_HEIGHT, self.Update) end)

local art1 = frame:CreateTexture(nil, "BORDER")
art1:SetTexture(0, 0, 0)
art1:SetPoint("TOPLEFT", scrollFrame, 0, 4)
art1:SetPoint("BOTTOMRIGHT", scrollFrame, 26, -4)

local art2 = frame:CreateTexture(nil, "BORDER", nil, -1)
art2:SetTexture(0.18, 0.18, 0.18)
art2:SetPoint("TOPLEFT", art1, -1, 1)
art2:SetPoint("BOTTOMRIGHT", art1, 1, -1)

local function onClick(self)
	if self:GetChecked() then
		PlaySound("igMainMenuOptionCheckBoxOn")
	else
		PlaySound("igMainMenuOptionCheckBoxOff")
	end
	
	local equipLoc = tabIDs[PanelTemplates_GetSelectedTab(frame)]
	local itemID = sortedItems[equipLoc][self:GetID() + FauxScrollFrame_GetOffset(scrollFrame)]
	if IsShiftKeyDown() then
		-- delete item from database and update scroll frame
		FashionData[equipLoc][itemID] = nil
		addon:Print(self:GetText().." removed from database.")
		addon:SortItems()
		scrollFrame:Update()
	else
		-- toggle visibility setting and reflect change
		FashionData[equipLoc][itemID] = not FashionData[equipLoc][itemID]
		addon:UpdateVisibility()
	end
end

local function createListItem()
	local btn = CreateFrame("CheckButton", nil, frame, "OptionsBaseCheckButtonTemplate")
	btn:SetSize(26, 26)
	btn:SetPushedTextOffset(0, 0)
	btn:SetScript("OnClick", onClick)
	btn.tooltipText = "Click to toggle visibility state.\nShift-click to remove."
	
	local text = btn:CreateFontString(nil, nil, "GameFontNormal")
	text:SetJustifyH("LEFT")
	text:SetPoint("LEFT", btn, "RIGHT", 0, 1)
	text:SetPoint("RIGHT", scrollFrame)
	btn:SetFontString(text)
	
	return btn
end

addon.itemList = {}

for i = 1, NUM_BUTTONS do
	local checkButton = createListItem()
	if i == 1 then
		checkButton:SetPoint("TOPLEFT", scrollFrame, 0, 4)
	else
		checkButton:SetPoint("TOP", addon.itemList[i - 1], "BOTTOM", 0, 6)
	end
	checkButton:SetID(i)
	addon.itemList[i] = checkButton
end

local function onClick(self)
	PanelTemplates_SetTab(frame, self:GetID())
	scrollFrame:Update()
end

local tab1 = CreateFrame("Button", "FashionFrameTab1", frame, "TabButtonTemplate")
tab1:SetID(1)
tab1:SetPoint("BOTTOMLEFT", scrollFrame, "TOPLEFT", 8, 7)
tab1:SetText("Helm")
tab1:SetScript("OnClick", onClick)
tab1.deselectedTextY = -6
PanelTemplates_TabResize(tab1, 0)
FashionFrameTab1Text:ClearAllPoints()

local tab2 = CreateFrame("Button", "FashionFrameTab2", frame, "TabButtonTemplate")
tab2:SetID(2)
tab2:SetPoint("LEFT", tab1, "RIGHT")
tab2:SetText("Cloak")
tab2:SetScript("OnClick", onClick)
tab2.deselectedTextY = -6
PanelTemplates_TabResize(tab2, 0)
FashionFrameTab2Text:ClearAllPoints()

PanelTemplates_SetNumTabs(frame, 2)
PanelTemplates_SetTab(frame, 1)

local closeButton = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
closeButton:SetPoint("TOPRIGHT", 0, -0)

local help = frame:CreateFontString(nil, nil, "GameFontNormalSmall")
help:SetPoint("BOTTOMLEFT", 12, 12)
help:SetText("Shift-click items to add")

if not ModifiedItemClickHandlers then
	ModifiedItemClickHandlers = {}
	
	local origHandleModifiedItemClick = HandleModifiedItemClick
	
	function HandleModifiedItemClick(link)
		if not link then
			return false
		end
		for i, v in ipairs(ModifiedItemClickHandlers) do
			if v(link) then
				return true
			end
		end
		return origHandleModifiedItemClick(link)
	end
end

tinsert(ModifiedItemClickHandlers, function(link)
	if frame:IsShown() then
		local itemID = link:match("item:(%d+)")
		local equipLoc = itemID and equipSlots[select(9, GetItemInfo(itemID))]
		if not equipLoc then
			addon:Print("Invalid item type. Please provide a helm or cloak item.")
		else
			local _, itemLink = GetItemInfo(itemID)
			itemID = tonumber(itemID)
			if FashionData[equipLoc][itemID] ~= nil then
				addon:Print(format("%s is already in the %s list.", itemLink, equipLoc))
			else
				FashionData[equipLoc][itemID] = true
				addon:Print(format("%s added to the %s list.", itemLink, equipLoc))
				addon:SortItems()
				local selectedTab = PanelTemplates_GetSelectedTab(frame)
				if tabIDs[selectedTab] ~= equipLoc then
					PanelTemplates_SetTab(frame, 3 - selectedTab)
				end
				scrollFrame:Update()
				addon:UpdateVisibility()
			end
		end
		return true
	end
end)

SLASH_FASHION1 = "/fashion"
SlashCmdList["FASHION"] = function()
	frame:SetShown(not frame:IsShown())
end

function addon:ADDON_LOADED(addon)
	if addon == "Fashion" then
		-- initiate variables if none existing
		FashionData = FashionData or {
			helm = {},
			cloak = {},
		}
		self:UnregisterEvent("ADDON_LOADED")
	end
end

function addon:PLAYER_LOGIN()
	helmID = self:GetVisibleItemID(headSlot)
	cloakID = self:GetVisibleItemID(backSlot)
	self:UpdateVisibility()
	self:SortItems()
end

local function onUpdate(self)
	self:SortItems()
	scrollFrame:Update()
	self:UnregisterEvent("GET_ITEM_INFO_RECEIVED")
	self:SetScript("OnUpdate", nil)
end

function addon:GET_ITEM_INFO_RECEIVED()
	self:SetScript("OnUpdate", onUpdate)
end

function addon:UpdateSlot(slot)
	-- update equipped items' IDs
	if slot == headSlot then
		helmID = self:GetVisibleItemID(slot)
	elseif slot == backSlot then
		cloakID = self:GetVisibleItemID(slot)
	else
		return
	end
	-- refresh helm/cloak visibility
	self:UpdateVisibility()
end

addon.PLAYER_EQUIPMENT_CHANGED = addon.UpdateSlot
addon.TRANSMOGRIFY_SUCCESS = addon.UpdateSlot

-- check if your helm is supposed to be shown or hidden every time your inventory changes
function addon:UpdateVisibility()
	local helmShown = FashionData.helm[helmID]
	local cloakShown = FashionData.cloak[cloakID]
	
	-- show/hide depending on which list item was in
	if helmShown ~= nil then
		ShowHelm(helmShown)
	end
	if cloakShown ~= nil then
		ShowCloak(cloakShown)
	end
end

function addon:GetVisibleItemID(slot)
	local isTransmogrified, _, _, _, _, visibleItemID = GetTransmogrifySlotInfo(slot)
	return isTransmogrified and visibleItemID or GetInventoryItemID("player", slot)
end

local function itemSorter(a, b)
	local itemA, itemB = GetItemInfo(a), GetItemInfo(b)
	if not (itemA and itemB) then
		addon:RegisterEvent("GET_ITEM_INFO_RECEIVED")
		return
	end
	return itemA < itemB
end

function addon:SortItems()
	for equipLoc, items in pairs(FashionData) do
		wipe(sortedItems[equipLoc])
		for id in pairs(items) do
			sortedItems[equipLoc][#sortedItems[equipLoc] + 1] = id
		end
		sort(sortedItems[equipLoc], itemSorter)
	end
end

function scrollFrame:Update()
	local equipLoc = tabIDs[PanelTemplates_GetSelectedTab(frame)]
	
	local size = #sortedItems[equipLoc]
	
	FauxScrollFrame_Update(self, size, NUM_BUTTONS, BUTTON_HEIGHT)
	
	local offset = FauxScrollFrame_GetOffset(self)
	
	for line = 1, NUM_BUTTONS do
		local button = addon.itemList[line]
		local lineplusoffset = line + offset
		if lineplusoffset <= size then
			local itemID = sortedItems[equipLoc][lineplusoffset]
			button:SetText(select(2, GetItemInfo(itemID)))
			button:SetChecked(FashionData[equipLoc][itemID])
			button:Show()
		else
			button:Hide()
		end
	end
end

function addon:Print(text)
	print("|cff00ff00Fashion:|r", text)
end